#include "drv_adc.h"
#include "hal_data.h"
#include <stdio.h>

#define LOG(FUNC, LINE)         printf("Function:%s\tLine:%d\r\n", FUNC, LINE)

static void ADCWaitConvCplt(void);
static int ADCDrvInit(struct ADCDev *ptDev);
static int ADCDrvRead(struct ADCDev *ptDev, unsigned short *buffer, unsigned short num);

static struct ADCDev gAdcDev = {
    .name = "ADC0",
    .channel = 5,
    .Init = ADCDrvInit,
    .Read = ADCDrvRead
};

static volatile bool adc_sample_cplt = false;

struct ADCDev *ADCGetDevice(void)
{
    return &gAdcDev;
}

static int ADCDrvInit(struct ADCDev *ptDev)
{
    if(NULL == ptDev)   return -1;
    if(5 == ptDev->channel)
    {
        /* 打开ADC设备完成通用初始化 */
        fsp_err_t err = g_adc5.p_api->open(g_adc5.p_ctrl, g_adc5.p_cfg);
        assert(FSP_SUCCESS == err);
        /* 配置ADC指令的通道完成初始化 */
        err = g_adc5.p_api->scanCfg(g_adc5.p_ctrl, g_adc5.p_channel_cfg);
        assert(FSP_SUCCESS == err);
        /* 打开ELC设备完成初始化 */
        err = g_elc.p_api->open(g_elc.p_ctrl, g_elc.p_cfg);
        assert(FSP_SUCCESS == err);
        /* 使能ELC的连接功能 */
        err = g_elc.p_api->enable(g_elc.p_ctrl);
        assert(FSP_SUCCESS == err);
        /* 打开DMA设备完成初始化 */
        err = g_transfer0.p_api->open(g_transfer0.p_ctrl, g_transfer0.p_cfg);
        assert(FSP_SUCCESS == err);
        /* 使能DMAC的ELC触发源 */
        err = g_transfer0.p_api->enable(g_transfer0.p_ctrl);
        assert(FSP_SUCCESS == err);
        /* 打开定时器设备完成初始化 */
        err = g_timer0.p_api->open(g_timer0.p_ctrl, g_timer0.p_cfg);
        assert(FSP_SUCCESS == err);
        /* 使能ADC的转换功能 */
        err = g_adc5.p_api->scanStart(g_adc5.p_ctrl);
        assert(FSP_SUCCESS == err);
    }
    
    return 0;
}

static int ADCDrvRead(struct ADCDev *ptDev, unsigned short *buffer, unsigned short num)
{
    if(NULL == ptDev || NULL == buffer || num <= 0)   return -1;
    if(5 == ptDev->channel)
    {
        /* 每次采样前将DMAC的目的地址和传输个数重置 */
        g_transfer0.p_cfg->p_info->p_dest = buffer;
        g_transfer0.p_cfg->p_info->length = num;
        fsp_err_t err = g_transfer0.p_api->reconfigure(g_transfer0.p_ctrl, g_transfer0.p_cfg->p_info);
        assert(FSP_SUCCESS == err);
        /* 开启定时器触发ADC采样 */
        err = g_timer0.p_api->start(g_timer0.p_ctrl);
        assert(FSP_SUCCESS == err);
        ADCWaitConvCplt();
        /* 采样结束后关闭定时器 */
        err = g_timer0.p_api->stop(g_timer0.p_ctrl);
        assert(FSP_SUCCESS == err);
    }
    
    return 0;
}

static void ADCWaitConvCplt(void)
{
    while(!adc_sample_cplt);
    adc_sample_cplt = false;
}


void adc5_dma_callback(dmac_callback_args_t * p_args)
{
    adc_sample_cplt = true;
}
